################################################################################
#
# date:     2007/01/17
# modified: 2007/03/19
#
# part of the RBazi-package 
#
################################################################################

checkParams <- function(params, separate=FALSE) {

	# init
	allowedMethods <- c("DivSta", "DivDwn", "DivAbr", "DivStd", "DivUp", "DivAuf", "DivGeo", "DivHar", "DivPMean","DivPow", "DivPot","DivUpw")
	# for districtwise or monoprop
	if (separate || is.element("housesize", names(params))) allowedMethods <- c(allowedMethods, "HaQgrR", "DrQgrR") 
	allowedBipropAlgorithms <- c("asmdpt", "asextr", "asrand", "ttinte", "ttflpt",
								 "ASmdpt", "ASextr", "ASrand", "TTinte", "TTflpt",
	                             "h_asmdpt_ttflpt", "h_asextr_ttflpt", "h_asrand_ttflpt",
	                             "ASmdpt|TTflpt", "ASextr|TTflpt", "ASrand|TTflpt",
	                             "h_asmdpt_ttinte", "h_asextr_ttinte", "h_asrand_ttinte",
	                             "ASmdpt|TTinte", "ASextr|TTinte", "ASrand|TTinte",
	                             "h_ipfp_ttinte", "h_ipfp_ttflpt",
	                             "IPFP|TTinte", "IPFP|TTflpt")
	
	
	# check votes / weights
	if (any(params$votes<0)) stop("Input weights / votes have to be nonnegative")
	# check minimum requirements
	if (!identical(params$minConstr, NA) && (length(params$minConstr) != length(params$votes))) stop("Minimum requirements do not correspond to votes.")
	if (!(identical(params$maxConstr, NULL) || identical(params$maxConstr, NA)) && (length(params$maxConstr) != length(params$votes))) stop("Maximum requirements do not correspond to votes.")
	if (!(identical(params$directSeats, NULL) || identical(params$directSeats, NA)) && (length(params$directSeats) != length(params$votes))) stop("Direct seats do not correspond to votes.")

	#check base
	if(is.element("base", names(params))){ #eg
	  if (params$base < 0) stop("Base seats have to be nonnegative")
	  if (!identical(params$base, NULL) && params$base*length(params$votes) > params$housesize) stop("Base seats greater than housesize")
	}
		
	if (is.element("housesize", names(params))) {	# monoprop
		if (any(params$housesize < 0)) stop("Housesize has to be nonnegative")
		if (!identical(params$minConstr, NA) && sum(params$minConstr) > params$housesize) stop("Minimum constraints greater than housesize")
		if (!identical(params$maxConstr, NA) && sum(params$maxConstr) < params$housesize) stop("Maximum constraints smaller than housesize")		
	}
	else { # district
		if (!separate) { 
			if (!is.null(params$appMethodOZ)) {
				if (length(params$appMethodOZ) > 1) stop("Only one extra rounding method for superapporitonment allowed.")
				if (is.element(params$appMethodOZ, c(allowedMethods, "HaQgrR", "DrQgrR"))) stop("Wrong rounding method for super apportionment.")
			}
			# check minimum requirements for superapportionment
			if (!all(is.na(params$minConstrSuper)) && !all(is.na(params$colM))) stop("Confusion on arguments minConstrSuper AND colM.")
			if (!all(is.na(params$minConstr))) {
				if (any(params$minConstr < 0)) stop("Minimum requirements have to be positive")
				if (any(rowSums(params$minConstr) > params$districtMagnitudes)) stop("Minimum requirements per row are larger than row marginals.")
			}	
			if (!all(is.na(params$minConstrSuper)) && any(params$minConstrSuper < 0)) stop("Minimum requirements have to be positive.")
			# check algorthims
			if (!all(is.element(params$algo, allowedBipropAlgorithms))) stop("Please check algorithms.")
			if (!all(is.na(params$colM))) {
				if (any(params$colM < 0)) stop("Party seats / col marginals have to be nonnegative")
				if (sum(params$districtMagnitudes) != sum(params$colM)) stop("Row and col marginals have to have the same sum.")
			}
		}
		if (any(params$districtMagnitudes < 0)) stop("District sizes / row marginals have to be positive.")
		
	}
	
	# check apportionment method(s)
	if (!all(is.element(params$appMethod, allowedMethods))) stop("Wrong rounding method.")
	if (params$appMethod == "DivSta" && is.null(params$pq)) stop("Specify paramter for stationary rounding")
	if (params$appMethod == "DivPMean" && is.null(params$pq)) stop("Specify parameter for power means rounding")
	

}
