################################################################################
#
# date:     2007/01/16
# modified: 2009/05/03
#           2018/03/01 wb
#           2018/03/03 EG						 
#
# corresponds to class rjBazi.ReadBaziFile 
#
################################################################################

readBazifile <- function(file, sorting="votes desc") {
	return(read.bazi(file, sorting=sorting)) 
}	

read.bazi <- function(file, sorting=c("alpha", "none", "votes desc", "votes asc")) {

	allowedSortingMethods = c("none", "alpha", "votes desc", "votes asc")
	if (length(sorting) != 1) sorting <- sorting[1]#stop("Please check sorting method.")
	match.arg(sorting)	
	
	if (!(length(grep("^/", file)) > 0 || length(grep("^[a-z]:", file))>0) ||
        length(grep("^~", file)) > 0) 
                file <- paste(getwd(), file, sep = "/")

	bazifile <- .jnew("rjBazi/ReadBazifile", file)
	
	if (.jcall(bazifile, returnSig="Z", "getOpenSucc")) {
		
		districtData <- .jcall(bazifile, "Z", "getDistrictData")
	
		appInfo <- list()
		
		# rounding methods, names
		appInfo$name <- .jcall(bazifile, "S", "getName")
		appInfo$appMethod <- .jcall(bazifile, "[S", "getMethods")
		if (is.element("DivSta", appInfo$appMethod)) {
			ind <- which(appInfo$appMethod == "DivSta")
			tmp <- .jcall(bazifile, "S", "getParams", as.integer(ind - 1))
			if (tmp == "simple")
                appInfo$pq <- .jcall(bazifile, "D", "getParam", as.integer(ind-1))
			else appInfo$pq <- tmp
			if (is.element("DivPot", appInfo$appMethod)) {
                warning(paste("Found DivPot and DivSta, removing DivPot from ",
                    "apportionment methods.", sep=""))
				appInfo$appMethod <-
                        appInfo$appMethod[-which(appInfo$appMethod=="DivPot")]
			}	
		}
		else if (is.element("DivPot", appInfo$appMethod)) {
			ind <- which(appInfo$appMethod == "DivPot")
			tmp <- .jcall(bazifile, "S", "getParams", as.integer(ind - 1))
			if (tmp == "simple") appInfo$pq <- .jcall(bazifile, "D", "getParam",
                as.integer(ind - 1))
			else appInfo$pq <- tmp
		}
		else appInfo$pq <- NA
		if (districtData)
                districtNames <- .jcall(bazifile, "[S", "getDistrictNames")
		partyNames <- .jcall(bazifile, "[S", "getPartyNames")
		if (districtData) {
			# district sizes
			appInfo$districtMagnitudes <- .jcall(bazifile, "[I", "getDistrictMagnitudes")
			names(appInfo$districtMagnitudes) <- districtNames
		}
		else {
			# house size
			appInfo$housesize <- .jcall(bazifile, "[I", "getHousesize")
			cols <- length(partyNames)
		}	
		# votes
		tmpVec <- as.numeric(.jcall(bazifile, "[D", "getVotes"))
		if (districtData) {
		
		# get district options
			rows <- length(districtNames)
			cols <- length(partyNames)
			
			appInfo$nzz <- .jcall(bazifile, "Z", "getNZZ")	
			appInfo$separate <- .jcall(bazifile, "Z", "getSeparate")

		
			if (length(tmpVec) != (rows * cols)) stop("Wrong dimension of votes")
			appInfo$votes <- matrix(tmpVec, nrow = rows, byrow = TRUE)
			rownames(appInfo$votes) <- districtNames
			colnames(appInfo$votes) <- partyNames
			# minimum requirements
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getMinConstr"))
			if (identical(tmpVec, -1)) appInfo$minConstr <- NA
			else {
				if (length(tmpVec) != (rows * cols)) stop("Wrong dimension of votes")
				appInfo$minConstr <- matrix(tmpVec, nrow = rows, byrow = TRUE)
				rownames(appInfo$minConstr) <- districtNames
				colnames(appInfo$minConstr) <- partyNames
			}
			# maximum requirements
			if (appInfo$separate) {
				tmpVec <- as.numeric(.jcall(bazifile, "[I", "getMaxConstr"))
				if (identical(tmpVec, -1)) appInfo$maxConstr <- NA
				else {
					if (length(tmpVec) != (rows * cols)) stop("Wrong dimension of votes")
					appInfo$maxConstr <- matrix(tmpVec, nrow = rows, byrow = TRUE)
					appInfo$maxConstr[appInfo$votes==0] <- 0
					rownames(appInfo$maxConstr) <- districtNames
					colnames(appInfo$maxConstr) <- partyNames
				}
			# direct seats	
				tmpVec <- as.numeric(.jcall(bazifile, "[I", "getDirectseats"))
				if (identical(tmpVec, -1)) appInfo$directSeats <- NA
				else {
					if (length(tmpVec) != (length(districtNames) * cols)) stop("Wrong dimension of direct setas")
					appInfo$directSeats <- matrix(tmpVec, nrow = rows, byrow = T)
					rownames(appInfo$directSeats) <- districtNames
					colnames(appInfo$directSeats) <- partyNames
				}	
			}	
			
			# minimum requirements for superapportionment
			if (!appInfo$separate) {
				appInfo$minConstrSuper <- as.numeric(.jcall(bazifile, "[I", "getMinConstrSuper"))
				ifelse(identical(appInfo$minConstrSuper, -1), appInfo$minConstrSuper <- NA, names(appInfo$minConstrSuper) <- partyNames)
			}
			
			# list combinations (!) only allowed for separate
			if (appInfo$separate) {
				tmpVec <- as.numeric(.jcall(bazifile, "[I", "getListCombinations"))
				if (identical(tmpVec, -1)) appInfo$listCombination <- NA
				else {
					if (length(tmpVec) != (rows * cols)) stop("Wrong dimension of list combinations")
					appInfo$listCombination <- matrix(tmpVec, nrow = rows, byrow = TRUE)
					rownames(appInfo$listCombination) <- districtNames
					colnames(appInfo$listCombination) <- partyNames
				}
			}
						
if (sorting == "alpha" || sorting == "votes desc" || sorting == "votes asc") {
				ifelse (sorting == "alpha" , tmpOrder <- order(colnames(appInfo$votes)),
					ifelse(sorting == "votes desc", ifelse(appInfo$nzz, 
			                                           tmpOrder <- order(colSums(diag(1/appInfo$districtMagnitudes)%*%appInfo$votes), decreasing = T), 
			                                           tmpOrder <- order(colSums(appInfo$votes), decreasing = T)),
			                ifelse(appInfo$nzz, 
			                       tmpOrder <- order(colSums(diag(1/appInfo$districtMagnitudes)%*%appInfo$votes), decreasing = F), 
			                       tmpOrder <- order(colSums(appInfo$votes), decreasing = F))
			         )
			  )
				appInfo$votes <- appInfo$votes[, tmpOrder]
				if (!identical(appInfo$minConstr, NA))
                        appInfo$minConstr <- appInfo$minConstr[, tmpOrder]
				if (appInfo$separate && !identical(appInfo$maxConstr, NA))
                        appInfo$maxConstr <- appInfo$maxConstr[, tmpOrder]
				if (appInfo$separate && !identical(appInfo$directSeats, NA))
                        appInfo$directSeats <- appInfo$directSeats[, tmpOrder]
				if (!appInfo$separate && !identical(appInfo$minConstrSuper, NA))
                        appInfo$minConstrSuper <- appInfo$minConstrSuper[tmpOrder]
				if (appInfo$separate && !identical(appInfo$listCombination, NA))
                        appInfo$listCombination <- appInfo$listCombination[, tmpOrder]
			}
			else if (sorting == "none") {
				# nothing to do
			}
			else stop("Never come here.")
			
			
			
			# to be sure that nothing else is in this variable
			appInfo$colM <- NA
		}
		else {
			if(length(tmpVec) != cols) stop("Partynames and votes do not fit.")
			appInfo$votes <- tmpVec
			names(appInfo$votes) <- partyNames
			# minimum requirements
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getMinConstr"))
			if (identical(tmpVec, -1)) appInfo$minConstr <- NA
			else {
				if (length(tmpVec) != cols) stop("Votes and min requs do not fit.")
				appInfo$minConstr <- tmpVec
				names(appInfo$minConstr) <- partyNames
			}
			# maximum requirements
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getMaxConstr"))
			if (identical(tmpVec, -1)) appInfo$maxConstr <- NA
			else {
				if (length(tmpVec) != cols) stop("Votes and min requs do not fit.")
				appInfo$maxConstr <- tmpVec
				appInfo$maxConstr[appInfo$votes == 0] <- 0
				names(appInfo$maxConstr) <- partyNames
			}
			# base from base+prop 
			appInfo$base <- .jcall(bazifile, "I", "getBmmBase")	
			
			# power flag indicating if base+prop(pwr) is set
			appInfo$isBmmPwr <- .jcall(bazifile, "Z", "getIsBmmPwr")	
			
			# global min/max from base+prop(pwr) (only relvant for power compromise;
			# for base+prop min/max requirement per party are used)
			appInfo$min <- .jcall(bazifile, "I", "getMin")	
			appInfo$max <- .jcall(bazifile, "I", "getMax")	
			
			# direct seats
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getDirectseats"))
			if (identical(tmpVec, -1)) appInfo$directSeats <- NA
			else {
				if (length(tmpVec) != cols) stop("Votes and directSeats do not fit.")
				appInfo$directSeats <- tmpVec
				names(appInfo$directSeats) <- partyNames
			}
			# naiv apportionment
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getNaiv"))
			if (identical(tmpVec, -1)) appInfo$naiv <- NA
			else {
				if (length(tmpVec) != cols) stop("Votes and naiv apportionment do not fit.")
				appInfo$naiv <- tmpVec
				names(appInfo$naiv) <- partyNames
			}

			# list combinations
			tmpVec <- as.numeric(.jcall(bazifile, "[I", "getListCombinations"))
			if (identical(tmpVec, -1)) appInfo$listCombination <- NA
			else {
				if (length(tmpVec) != cols) stop("Votes and list combinations do not fit.")
				appInfo$listCombination <- tmpVec
				names(appInfo$listCombination) <- partyNames
			}
			# sorting
			if (sorting == "alpha" || sorting == "votes desc" || sorting == "votes asc") {
				ifelse(sorting == "alpha", 
					tmpOrder <- order(names(appInfo$votes)),
					ifelse(sorting == "votes desc", tmpOrder <- order(appInfo$votes, decreasing = TRUE),
						tmpOrder <- order(appInfo$votes, decreasing = FALSE)
					)
				)
				appInfo$votes <- appInfo$votes[tmpOrder]
				if (!identical(appInfo$minConstr, NA))
                        appInfo$minConstr <- appInfo$minConstr[tmpOrder]
				if (!identical(appInfo$maxConstr, NA))
                        appInfo$maxConstr <- appInfo$maxConstr[tmpOrder]
				if (!identical(appInfo$directSeats, NA))
                        appInfo$directSeats <- appInfo$directSeats[tmpOrder]
				if (!identical(appInfo$naiv, NA))
                        appInfo$naiv <- appInfo$naiv[tmpOrder]
				if (!identical(appInfo$listCombination, NA))
                        appInfo$listCombination <- appInfo$listCombination[tmpOrder]

			}
			else if (sorting == "none") {
				# nothing to do
			}
			else stop("Never come here.")
			
		}
		
		# remove reference to Java-object
		rm(bazifile)
		#.jcall(bazifile, returnSig = "V", "finalize")
		
		class(appInfo) <- "RBaziData"
		return(appInfo)
	}
	else {
		stop(.jcall(bazifile, returnSig = "S", "getErrorMsg"))
	}	
	
}