\name{monopropApp}

\alias{monopropApp}

\title{Adapter module for monoproportional apportionment using BAZI}

\description{Computes a monoproportional Apportionment}

\usage{
monopropApp(housesize, votes, appMethod="DivStd", pq=NULL, minConstr=NA, maxConstr = NA,
            directSeats = NA, sorted=F, resolveTies=F, listCombination = NA, name = NULL, base = 0)}

\arguments{
	\item{housesize}{Number of seats to be apportioned}
	\item{votes}{Vector with input weights, e.g. votes for parties}
	\item{appMethod}{Apportionment method(s) to be used for apportionment, see details}
	\item{pq}{\eqn{\in [0,1]}{in [0,1]} for stationary rounding \code{"DivSta"},
		\eqn{\in R}{in R} for power means rounding \code{"DivPMean"}}
	\item{minConstr}{Minimum requirements, forces \code{apporitonment >= minConstr}}
	\item{maxConstr}{Maximum constraints, forces \code{apportionment <= maxConstr}}
	\item{directSeats}{Vector of direct seats for mixed member systems}
	\item{sorted}{Sort parties in decreasing order of number of votes}
	\item{resolveTies}{Resolve ties if there are multiple apportionments}
	\item{listCombination}{Vector indicating belonging to a list combination, 
		$0$ indicates not member of a list combination}
	\item{name}{Name for the apportionment problem}
	\item{base}{Number of base seats}
}

\details{
For the \kbd{appMethod} argument the different following rounding methods are valid:
rounding down (\code{"DivDwn"}) to the last integer, standard rounding 
(\code{"DivStd"}) in the "usual way", geometric rounding (\code{"DivGeo"}),
harmonic rounding (\code{"DivHar"}), rounding up (\code{"DivUp"}) to the next
integer, general stationary rounding (\code{"DivSta"}), and power means
rounding (\code{"DivPMean"}). Additionaly, there are two quota methods
implemented: Hare quota with greatest remainders (\code{"HaQgrR"}) and
Droop quota with greates remainders (\code{"DrQgrR"}).
}

\value{
A list with components
	\item{votes}{input votes (sorted) and then recursive}
	\item{apportionment}{}
	\item{ties}{}
	\item{pt}{}
}

\seealso{\code{\link{computeApportionment}}}

\references{
	\url{http://www.uni-augsburg.de/bazi}.
	
	Maier, S. and Pukelsheim, F. (2007) \emph{BAZI - A Free Program for Proportional
	Representation Apportionment}, Preprint No. 42, Institute for Mathematics, University
	of Augsburg.
}	

\examples{
# Data of the German federal elections 2009
votes <- c(11824794, 9988843, 6313023, 5153884, 4641197, 2830210)
names(votes) <- c("CDU", "SPD", "FDP", "Die Linke", "Gruene", "CSU")
directSeats <- c(173, 64, 0, 16, 1, 45)
names(directSeats) <- names(votes)
housesize <- 598
# Apportionment without direct seats
monopropApp(housesize=housesize, votes=votes, appMethod="DivStd")
# Apportionment with direct seats
monopropApp(housesize=housesize, votes=votes, appMethod="DivStd", directSeats=directSeats)

}

\keyword{math}
